/***************************************************************************
 *                                                                         *
 *  Copyright 1995 Symbios Logic Incorporated.  All rights reserved.       *
 *                                                                         *
 * This software was written by Symbios Logic Inc. to develop and test new *
 * products.  Symbios Logic assumes no liability for its use. This software*
 * is released to the public domain to illustrate certain programming      *
 * techniques for testing the 53c8xx chips                                 *
 *                                                                         *
 ***************************************************************************/

/*           Name:  8XX_PCI.C
 *          Title:  Validation test code 53C8XX PCI tools
 *      $Workfile:  $
 *      $Revision:  $
 *       $Modtime:  $
 *     Programmer:  Symbios Logic
 *  Creation Date:  11/29/95
 *
 *  Version History
 *  ---------------
 *
 *    Date    Who?  Description
 *  --------  ----  -------------------------------------------------------
#BeginRevision
 *  03/06/96  SYM   - Fixed up devid2str function to eliminate problems when
 *						  an invalid device id version id sequence was given.  Also
 *						  added support for all known 8XX devices
 *						  - Fixed problem with multiple devices of the same type
 *						  being found.  Only the first device was found and no others
 *						  Was not setting the device id in the next array location
 *						  when looping after finding the first device.
 *						  - Added /n's to all Writelog calls
 *						  - Made general formatting changes and documentation
 *						  additions
 *
#EndRevision
 *
 *  11/29/95  SYM  Initial Revision.
 *
#BeginDescription
 *
 *  This file contains functions for manipulating 53C8XX pci devices
 *  These functions are specific to the 53C8XX chips and their PCI
 *  configurations and functionality.  It would be possible though to
 *  relatively easily port some of these functions over to another
 *  hardware platform.
 *
#EndDescription
 *
 *-------------------------------------------------------------------------
 *
 *  $Header: $
 *
 */


#include "valtypes.h"
/* the following two #includes must go in this order */
#include "pci.h"
#include "8xx_pci.h"
#include "gen_tool.h"
#include <stdio.h>
#include <conio.h>

/* Local Global Variables */
static VUINT ID_Array[] = {          /* used to scan for 8XX PCI devices */
	C810_DEVICE_ID, C815_DEVICE_ID,
	C820_DEVICE_ID, C825_DEVICE_ID,
	C810AP_DEVICE_ID, C860_DEVICE_ID,
	C875_DEVICE_ID, C885_DEVICE_ID, 
	C895_DEVICE_ID, C896_DEVICE_ID,
	NULL_DEVICE_ID };

/* Function Prototypes */
VCHAR *C8XX_DevID2Str(VINT id, VUBYTE rev_id);
VINT C8XX_BuildPCIDeviceList(pcidev_record *PCIDevice[], VINT Max_PCIDevices);
VINT C8XX_ChoosePCIDevice(pcidev_record *PCIDevice[], VINT NumPCIDevices);
VUINT C8XX_GetMemBase(pcidev_record *PCIDevice);
VUINT C8XX_GetIOBase(pcidev_record *PCIDevice);
VUINT C8XX_GetRAMBase(pcidev_record *PCIDevice);
void PCI_print_config(pcidev_record *PCIDevice);

/******************************************************************************/
/* Function: C8XX_DevID2Str
	Purpose : Returns 53C8XX Device Name string given a 53C8XX Device ID
	Input:  Valid 53C8XX device ID
	Output: Descriptor string of 53C8xx device
				"Unknown Symbios SCSI Device" if the ID pass in is not known
	Assumptions: None
	Other functions called: None
*/
/******************************************************************************/

VCHAR *C8XX_DevID2Str(VINT id, VUBYTE rev_id)
{

	switch (id) 
	{
		case C810_DEVICE_ID: 
		{
			/* Could be 810 or 810a, check revID register to tell */
			if ((rev_id & 0xF0) == C810_VERSION_ID) return("SYM 53C810");
			if ((rev_id & 0xF0) == C810A_VERSION_ID) return("SYM 53C810A");
			if ((rev_id & 0xF0) == C810ALV_VERSION_ID) return("SYM 53C810ALV");
			break;
		}
		case C815_DEVICE_ID:
		{        
			if ((rev_id & 0xF0) == C815_VERSION_ID) return("SYM 53C815");
			break;
		}
		case C820_DEVICE_ID:
		{        
			if ((rev_id & 0xF0) == C820_VERSION_ID) return("SYM 53C820");
			break;
		}
		case C825_DEVICE_ID: 
		{
			/* Could be 825 or 825a, check revID register to tell */
			if ((rev_id & 0xF0) == C825_VERSION_ID) return("SYM 53C825");
			if ((rev_id & 0xF0) == C825A_VERSION_ID) return("SYM 53C825A");
			break;
		}
		case C810AP_DEVICE_ID:
		{        
			if ((rev_id & 0xF0) == C810AP_VERSION_ID) return("SYM 53C810A Proto");
			break;
		}
		case C860_DEVICE_ID:
		{        
			if ((rev_id & 0xF0) == C860_VERSION_ID) return("SYM 53C860");
			if ((rev_id & 0xF0) == C860LV_VERSION_ID) return("SYM 53C860LV");
			break;
		}
		case C875_DEVICE_ID:
		{
			if ((rev_id & 0xF0) == C875_VERSION_ID) return("SYM 53C875");
			break;
		}
		case C885_DEVICE_ID:
		{        
			if ((rev_id & 0xF0) == C885_VERSION_ID) return("SYM 53C885");
			break;
		}
		case C895_DEVICE_ID:
		{        
			if ((rev_id & 0xF0) == C895_VERSION_ID) return("SYM 53C895");
			break;
		}
		case C896_DEVICE_ID:
		{        
			if ((rev_id & 0xF0) == C896_VERSION_ID) return("SYM 53C896");
			break;
		}
	}
	return("Unknown Symbios SCSI Device");
}


/******************************************************************************/
/* Function: BuildPCIDeviceList
	Purpose : Builds a list of 53C8XX devices found in the system
	Input:  Pointer to an array of pcidev_record structures and a limit
			  Max_PCIDevices indicating how big the pcidev_record array is.
	Output: This function fills the PCIDevice data structure by calling
				the PCI_FindSYMDevice function for all possible ID's.
				If Max_PCIDevices is reached then the function returns immediately
				with a return value = Max_PCIDevices, otherwise it returns the
				number of devices found.
	Assumptions: That enough PCIDevice structures have been allocated to fill in
					 Max_PCIDevices structures
	Other functions called: PCI_FindSYMDevice
*/
/******************************************************************************/

VINT C8XX_BuildPCIDeviceList(pcidev_record *PCIDevice[], VINT Max_PCIDevices)

{
	VINT NumPCIDevices;		/* Actual number of devices found */
	VINT i;                 /* Counting variable */

	if (!Max_PCIDevices) {          /* Ooops!!! didn't allocate any device space */
		return(0);
	}

	NumPCIDevices = 0;
	i = 0;

	/* look for devices */
	while (ID_Array[i] != NULL_DEVICE_ID)
	{
		PCIDevice[NumPCIDevices]->device_id = ID_Array[i];
		PCIDevice[NumPCIDevices]->device_index = 0;
		/* find all devices with a particular device id */
		while (PCI_FindSYMDevice(PCIDevice[NumPCIDevices]) != PCI_NO_DEVICE_FOUND)
		{
			NumPCIDevices++;
			/* did we run out of space to store devices, if yes return */
			if (NumPCIDevices == Max_PCIDevices) return(NumPCIDevices+1);
			/* bump the index to look for more of this device */
			PCIDevice[NumPCIDevices]->device_index =
											PCIDevice[NumPCIDevices-1]->device_index+1;
			/* set the next array item to the same device id */
			PCIDevice[NumPCIDevices]->device_id =
											PCIDevice[NumPCIDevices-1]->device_id;
		}
		i++;
	}

	return(NumPCIDevices);

}

/******************************************************************************/
/* Function: C8XX_ChoosePCIDevice
	Purpose : Allows the selection of a particular 8xx device in the system
				 Displays all devices and prompts for a selection
	Input:  Array of pcidev_records (for all devices in the system) and the
			  actual number of devices in the system
	Output: Returns the index of the selected device used to index into the
				PCIDevice array
	Assumptions: That NumPCIDevices gives the proper number of devices in the
					 PCIDevice array
	Other functions called: C8XX_DeviceID2Str
*/
/******************************************************************************/

VINT C8XX_ChoosePCIDevice(pcidev_record *PCIDevice[], VINT NumPCIDevices)

{
	VINT i;
	VINT device_select = -1;

	/* print devices from the PCIDevice array */
	for (i=0; i<NumPCIDevices; i++)
		fprintf(stdout, "%d: %s Bus #:%X Device #:%X Function :%X\n", i,
							C8XX_DevID2Str(PCIDevice[i]->device_id, PCIDevice[i]->rev_id),
							PCIDevice[i]->bus_num, PCIDevice[i]->device_num>>3,
							PCIDevice[i]->function);

	fprintf(stdout, "\n");

	/* prompt for device selection */
	while (device_select == -1)
	{
		fprintf(stdout, "Choose a device: ");
		scanf("%d", &device_select);
		fflush(stdin);
		if ((device_select < 0) || (device_select > NumPCIDevices-1))
			device_select = -1;
	}

	return(device_select);
}

/******************************************************************************/
/* Function: GetMemBase
	Purpose : Fills in the PCIDevice structure with the Memory Base address
				 of the 53C8XX device
	Input: Pointer to Structure pcidev_record PCIDevice which indicates which
			 device to get the Memory base address for
	Output: Fills in the PCIDevice->base_addr1 field and returns C8XX_GOOD status
			  if the value read is non-zero.  Returns C8XX_NO_BASE if the value
			  read is zero
	Assumptions: Pointer to device structure is valid.
	Other functions called: PCI_GetConfigRegister
*/
/******************************************************************************/

VUINT C8XX_GetMemBase(pcidev_record *PCIDevice)
{

	/* Read config register that contains the Register Memory Base address */
	PCIDevice->base_addr1 = PCI_GetConfigRegister(PCIDevice, C8XX_CONFIG_REG_MEMB);

	if (PCIDevice->base_addr1 & 0xFFFFFFFCl) return(C8XX_GOOD);
	else return(C8XX_NO_BASE); /* mem base address was not programed by system */

}

/******************************************************************************/
/* Function: GetIOBase
	Purpose : Fills in the PCIDevice structure with the IO Base address
				 of the 53C8XX device
	Input: Pointer to Structure pcidev_record PCIDevice which indicates which
			 device to get the IO base address for
	Output: Fills in the PCIDevice->base_addr2 field and returns C8XX_GOOD status
			  if the value read is non-zero.  Returns C8XX_NO_BASE if the value
			  read is zero
	Assumptions: Pointer to device structure is valid.
	Other functions called: PCI_GetConfigRegister
*/
/******************************************************************************/

VUINT C8XX_GetIOBase(pcidev_record *PCIDevice)
{
	/* Read config register that contains the Register IO Base address */
	PCIDevice->base_addr2 = PCI_GetConfigRegister(PCIDevice, C8XX_CONFIG_REG_IOB) &
								0xFFFFFFFEl;

	if(PCIDevice->base_addr2 & 0xFFFFFFFCl) return(C8XX_GOOD);
	else return(C8XX_NO_BASE); /* IO base address was not programmed by system */

}

/******************************************************************************/
/* Function: GetRAMBase
	Purpose : Fills in the PCIDevice structure with the SCRIPT RAM Base address
				 of the 53C825A or 875 device
	Input: Pointer to Structure pcidev_record PCIDevice which indicates which
			 device to get the SCRIPT RAM base address for
	Output: Fills in the PCIDevice->base_addr3 field and returns C8XX_GOOD status
			  if the value read is non-zero.  Returns C8XX_NO_BASE if the value
			  read is zero
	Assumptions: Pointer to device structure is valid and the device is a 53C825A
					 or a 53C875.
	Other functions called: PCI_GetConfigRegister
*/
/******************************************************************************/

VUINT C8XX_GetRAMBase(pcidev_record *PCIDevice)
{
	/* Read config register that contains the SCRIPT RAM Base address */
	PCIDevice->base_addr3 = PCI_GetConfigRegister(PCIDevice, C8XX_CONFIG_REG_RAMB);

	if(PCIDevice->base_addr3 & 0xFFFFFFFCl) return(C8XX_GOOD);
	else return(C8XX_NO_BASE); /* RAM base address was not programmed by system */

}

/******************************************************************************/
/* Function: PCI_print_config
	Purpose : To output configurtion information about a PCI device
	Input: Pointer to Structure pcidev_record PCIDevice.  This structure
			 contains the config information to be output.
	Output: PCI config information either to the screen or to a file
	Assumptions: none
	Other functions called: sprintf to format string for Writelog
									WriteLog to output information to file or screen
*/
/******************************************************************************/

void PCI_print_config(pcidev_record *PCIDevice)
{

	VCHAR string[50];

	sprintf(string,"\nDevice Name: %s\n", C8XX_DevID2Str(PCIDevice->device_id,
													PCIDevice->rev_id));
	WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);
	sprintf(string,"Device ID: %02X\n", PCIDevice->device_id);
	WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);
	sprintf(string,"Rev ID: %02X\n", PCIDevice->rev_id);
	WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);
	sprintf(string,"Bus #: %02X\n", PCIDevice->bus_num);
	WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);
	sprintf(string,"Device #: %02X\n", PCIDevice->device_num >> 3);
	WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);
	sprintf(string,"Device Function: %02X\n", PCIDevice->function);
	WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);
	sprintf(string,"Device Index: %d\n", PCIDevice->device_index);
	WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);
	sprintf(string,"Cache Line Size: %02X\n", PCIDevice->CLS);
	WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);
	sprintf(string,"Latency Timer: %02X\n", PCIDevice->latency);
	WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);
	sprintf(string,"Command Reg: %04X\n", PCIDevice->command);
	WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);
	sprintf(string,"IO Base: %04X\n", PCIDevice->base_addr2);
	WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);
	sprintf(string,"RAM Base: %08lX\n", PCIDevice->base_addr3);
	WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);
	sprintf(string,"ROM Base: %08lX\n", PCIDevice->ROM_base_addr);
	WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);
	sprintf(string,"Interrupt Line: %02X\n", PCIDevice->intl);
	WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);
	sprintf(string,"Interrupt Pin: %02X\n", PCIDevice->intp);
	WriteLog(LOG_PASS, LOG_DETAIL, string, NULL);

}
